
`define EOF 32'hFFFF_FFFF 
`define NULL 0
`timescale 1 ns / 10 ps


module tb_b3g;

parameter [3:0] 
  XCOM_IDL  = 4'b0000,
  XCOM_FRQ  = 4'b0001,
  XCOM_UPL  = 4'b0010,
  XCOM_COL  = 4'b0011,
  XCOM_OPT  = 4'b0100;

//instance i/o
reg fpga_reset,clk_osc;
wire rclk,adc_reset,calibrate,ren,wen,sdata,sclk,ecm_fsr,calDlyScs_;
reg ff,ef,wensync,ovrange,drdy1,drdy2;
reg [7:0] da,db;
wire [15:0] cy_data_p; //bidirectional bus
reg cy_req_p,load_state_p,mstr_sdata_p,mstr_sclk_p,read,capture,trig_in;
reg strobe_ip,strobe_in;
reg [3:0] state_p;
wire [4:1] test_p,test_n;
wire cy_valid_p,dc_select,pll_le,ext_clk_sel,nADC_1V9_en,diff_sel;
wire [7:0] dqa_p,dqa_n,dqb_p,dqb_n;
wire fifo_full,fifo_empty,strobe_q_p,strobe_q_n,upload,sample,trig_o,ovr_led,idle,power_down,clk_active;

//sim variables
reg cy_doe,cy_clk;
reg [15:0] cy_data;
integer sfile,r,i;
reg [80*8:1] str1,str2,str3;
reg [16*8:1] cmd;

Big3Gig i_b3g(.fpga_reset(fpga_reset),
	      .clk_osc(clk_osc),
	      .rclk_o(rclk),
	      .rclk_in_p(rclk),
	      .adc_reset(adc_reset),
	      .calibrate(calibrate),
	      .wensync(wensync),
	      .ovrange(ovrange),
	      .ef(ef),
	      .ff(ff),
	      .ren(ren),
	      .wen(wen),
	      .drdy1(drdy1),
	      .drdy2(drdy2),
	      .da(da),
	      .db(db),
	      .sdata(sdata),
	      .sclk(sclk),
	      .ecm_fsr(ecm_fsr),
	      .calDlyScs_(calDlyScs_),
	      .cy_data_p(cy_data_p),
	      .cy_req_p(cy_req_p),
	      .load_state_p(load_state_p),
	      .state_p(state_p),
	      .mstr_sdata_p(mstr_sdata_p),
	      .mstr_sclk_p(mstr_sclk_p),
	      .cy_valid_p(cy_valid_p),
	      .read(read),
	      .capture(capture),
	      .trig_in(trig_in),
	      .strobe_ip(strobe_ip),
	      .strobe_in(strobe_in),
	      .dc_select(dc_select),
	      .pll_le(pll_le),
	      .ext_clk_sel(ext_clk_sel),
	      .nADC_1V9_en(nADC_1V9_en),
	      .diff_sel(diff_sel),
	      .test_p(test_p),
	      .test_n(test_n),
	      .dqa_p(dqa_p),
	      .dqa_n(dqa_n),
	      .dqb_p(dqb_p),
	      .dqb_n(dqb_n),
	      .fifo_full(fifo_full),
	      .fifo_empty(fifo_empty),
	      .strobe_q_p(strobe_q_p),
	      .strobe_q_n(strobe_q_n),
	      .upload(upload),
	      .sample(sample),
	      .trig_o(trig_o),
	      .ovr_led(ovr_led),
	      .idle(idle),
	      .power_down(power_down),
	      .clk_active(clk_active)
               );


assign cy_data_p = (cy_doe) ? cy_data : 32'hZ;


//-----------------------------------------------------------------------
initial     begin : file_block
  $timeformat(-9,3," ns",4);

  sfile = $fopen("bgscr.txt","r");
  if (sfile == `NULL) begin
	$display("bgscr file NOT Found");
	disable file_block;
	$stop;
  end
  else $display("bgscr file open, sfile = %h",sfile);

  ff = 1'b0;
  ef = 1'b1;

  @(posedge fpga_reset);
  @(negedge fpga_reset) $display("FPGA Reset Done time = %t",$time); 

  r = 10;			//kill 10 clocks
  while (r) begin
	@(posedge cy_clk);
	r = r-1;
  end

  while (1) begin

	  cmd = "  ";
	  r = $fgets(str1,sfile);
   	  r = $sscanf(str1, "%s", cmd);

 	  	case (cmd)

			"wtclk":    begin
   	                      r = $sscanf(str1, "%s %d", str2,i);
                          while (i) begin
	                        @(posedge cy_clk);
	                        i = i-1;
				          end
				        end

			"echo":     begin
					      r = $sscanf(str1, "%s %s", str2,str3);
					      $display("%s",str3);
					    end

			"pr_time":	$display("%t",$time);

			"oprg_wr":  begin	
					      r = $sscanf(str1, "%s %h", str2,i);
						  @(posedge cy_clk)
						  #2
						  cy_data = i[15:0];
						  cy_doe  = 1'b1;
						  state_p = XCOM_OPT;
						  @(posedge cy_clk)
						  #2
						  load_state_p = 1'b1;
						  @(posedge cy_clk);
						  @(posedge cy_clk)
						  #2
						  load_state_p = 1'b0;
						  state_p      = XCOM_IDL;
						  cy_doe       = 1'b1;
						end

			"collect":  begin	
						  state_p = XCOM_COL;
						  @(posedge cy_clk)
						  #2
						  load_state_p = 1'b1;
						  @(posedge cy_clk);
						  @(posedge cy_clk)
						  #2
						  load_state_p = 1'b0;
						  state_p      = XCOM_IDL;

						  @(posedge rclk);
						  @(posedge rclk)
						  #2 ef = 1'b0;

                          r = 100;			//kill 100 rclks
                          while (r) begin
                            @(posedge rclk);
	                        r = r-1;
                          end

						  #2 ff = 1'b1;

			            end

			"//":       begin end  //comment

			"done":     begin
                          r = 50;			//kill 10 cy_clocks
                          while (r) begin
                            @(posedge cy_clk);
	                        r = r-1;
                          end
						  $display("Done Found at %t...Quitting",$time);
						  $fclose(sfile);
						  $stop;
					    end

		    default:    begin
					  	  $display("Unknown cmd '%0s'", cmd);
						end

 		endcase
  end // while not EOF

end // initial                        

//-----------------------------------------------------------------------


//////////////////////////////////////////////////////////////

initial				//initial state block
  begin
		  fpga_reset	= 0;

		  wensync       = 0;
		  ovrange       = 0;
		  drdy1         = 0;
		  drdy2         = 0;
		  da            = 8'h0;
		  db            = 8'h0;
		  cy_req_p      = 0;
		  load_state_p  = 0;
		  mstr_sdata_p  = 0;
		  mstr_sclk_p   = 0;
		  read          = 0;
		  capture       = 0;
		  trig_in       = 0;
		  strobe_ip     = 0;
		  strobe_in     = 0;
		  state_p       = 4'h0;

		  cy_doe        = 1'b0;
		  cy_data       = 8'hZ;
#10
		  fpga_reset	= 1;
#900
		  fpga_reset	= 0;
  end
//////////////////////////////////////////////////////////////

initial //clock gen clk_osc
  begin
    clk_osc = 0;
    #5 forever #5 clk_osc = ~clk_osc;
  end

initial //clock gen cy_clk
  begin
    cy_clk = 0;
    #11 forever #11 cy_clk= ~cy_clk;
  end



endmodule

