//This is for the buffered version i.e. the 8B3000 // //Design Notes...
//da is d1
//db is d2
//time sequence of bytes is:  db_r,da_r,db_f,da_f
//adc data gets written into a 32 bit wide register and onto a 32 bit wide (on input) fifo
//reg/fifo bus definition is as follows
//d2_r[7:0] <=  db_r
//d2_r[15:8] <= da_r
//d2_r[23:16] <= db_f
//d2_r[31:24] <= da_f
//
//
  
`timescale 1 ns / 10 ps

module Big3Gig(fpga_reset,clk_osc,rclk_o,rclk_in_p,
	       adc_reset,calibrate,wensync,ovrange,ef,ff,
	       ren,wen,drdy1,drdy2,da,db,sdata,sclk,ecm_fsr,calDlyScs_,
	       cy_data_p,cy_req_p,load_state_p,state_p,mstr_sdata_p,mstr_sclk_p,cy_valid_p,
	       read,capture,trig_in,strobe_ip,strobe_in,
	       dc_select,pll_le,ext_clk_sel,nADC_1V9_en,diff_sel,
	       test_p,test_n,dqa_p,dqa_n,dqb_p,dqb_n,fifo_full,fifo_empty,strobe_q_p,strobe_q_n,
	       upload,sample,trig_o,ovr_led,idle,power_down,clk_active
               );
input clk_osc,fpga_reset,rclk_in_p;
input ef,ff,drdy1,drdy2,wensync,ovrange;  //from adc
input [7:0] da,db;		//from adc
output adc_reset,rclk_o,ren,wen,calibrate,ecm_fsr,power_down; //to adc
output sdata,sclk,calDlyScs_;  //to ADC - serial port

inout [15:0] cy_data_p;  //cypress ports b and d
input cy_req_p,load_state_p,mstr_sdata_p,mstr_sclk_p;  //from cypress
input [3:0] state_p;  //from cypress
output cy_valid_p;  //to cypress

output dc_select,pll_le,ext_clk_sel,nADC_1V9_en,diff_sel;  //GPO
output upload,sample,trig_o,ovr_led,idle,clk_active; //leds

input read,capture,trig_in,strobe_ip,strobe_in;
output [4:1] test_p,test_n;
output [7:0] dqa_p,dqa_n,dqb_p,dqb_n;
output fifo_full,fifo_empty,strobe_q_p,strobe_q_n;

wire clk50,clk100,rclk_o;

reg [15:0] cy_data_i;
reg [15:0] cy_data_o;

reg cy_req,load_state;
wire cy_data_oe;
reg [3:0] state;

reg [15:0] op_reg;

reg [7:0] da_r,db_r;
reg [15:0] d2_r;

reg [2:0] sm_frq_cnt;
reg [15:0] frq_rg0,frq_rg1;
reg frq_rg0_vld,frq_rg1_vld,frq_cnt_on,frq_cnt_dn;

reg wen,ren,ren1,ren2,ren3,ren4,ren5;
reg ef_r,ef_r1,ef_r2,ff_r,ovr_ff;
reg [2:0] sm_coll;
wire [15:0] fifo_q;

reg fifo_q_vld;
reg [2:0] sm_upl;

reg [24:0] clk_led_cntr;

wire rclk,vclk,fifo_ren,fifo_wen,fifo_rst;

parameter [3:0] 
  XCOM_IDL     = 4'b0000,
  XCOM_FRQ     = 4'b0001,
  XCOM_UPL     = 4'b0010,
  XCOM_COL     = 4'b0011,
  XCOM_OPT     = 4'b0100,
  XCOM_RST_FF  = 4'b1110;



parameter [2:0]
  SM_FRQ_IDL = 3'h0,
  SM_FRQ_ST1 = 3'h1,
  SM_FRQ_ST2 = 3'h2,
  SM_FRQ_ST3 = 3'h3,
  SM_FRQ_ST4 = 3'h4;

parameter [2:0]
  SM_COLL_IDL = 3'h0,
  SM_COLL_ST1 = 3'h1,
  SM_COLL_ST2 = 3'h2,
  SM_COLL_ST3 = 3'h3,
  SM_COLL_ST4 = 3'h4;

parameter [2:0]
  SM_UPL_IDL = 3'h0,
  SM_UPL_ST1 = 3'h1,
  SM_UPL_ST2 = 3'h2,
  SM_UPL_ST3 = 3'h3,
  SM_UPL_ST4 = 3'h4;


ClkBlk i_ClkBlk(.fpga_reset(fpga_reset),
	    .clk_osc(clk_osc),
	    .rclk_in_p(rclk_in_p),
		.rclk_set({op_reg[10],op_reg[6],op_reg[3]}),
		.clk50(),
		.clk100(clk100),
		.rclk(rclk),
		.vclk(vclk),
		.clk_osc_lckd()
	       );

assign rclk_o = vclk;

 fifo_2kx16_dc i_fifo_r(  //symmetric FIFO
	.din(d2_r[15:0]),
	.rd_clk(clk100),
//	.rd_en((sm_upl == SM_UPL_ST1) & cy_req),
	.rd_en(fifo_ren),
	.rst(fifo_rst),
	.wr_clk(rclk),
	.wr_en(fifo_wen),
	.dout(fifo_q[15:0]),
	.empty(fifo_empty),
	.full(fifo_full)
       );
assign fifo_wen =	ren5 & ~ef_r2;
assign fifo_ren = ( (sm_upl == SM_UPL_ST1) & cy_req);
assign fifo_rst = ({load_state_p,state_p[3:0]} == {1'b1,XCOM_RST_FF});

/* 
 b3000_fifo i_xl_fifo(  //asymmetric FIFO
	.din(d_r2_r),
	.rd_clk(rclk),
	.rd_en((sm_upl == SM_UPL_ST1) & cy_req),
	.rst(fpga_reset),
	.wr_clk(rclk),
	.wr_en(ren5 & ~ef_r2),
	.dout(fifo_q),
	.empty(fifo_empty),
	.full(fifo_full)
       );
*/

always @(posedge fpga_reset or posedge rclk) //capture adc data on rising edge

  if (fpga_reset) begin
    da_r <= 8'h0;
    db_r <= 8'h0;
  end else
  begin
    da_r <= da;
    db_r <= db;
  end

always @(posedge fpga_reset or posedge rclk) //pipeline adc data, also ef and ff

  if (fpga_reset) begin
    d2_r        <= 16'h0;
    ef_r        <= 1'b0;
    ef_r1       <= 1'b0;
    ef_r2       <= 1'b0;
    ff_r        <= 1'b0;
  end else
  begin
    d2_r[7:0]   <= #1  db_r; //earliest sample
    d2_r[15:8]  <= #1  da_r;
    ef_r        <= #1  ef;
    ef_r1       <= #1  ef_r;
    ef_r2       <= #1  ef_r1;
    ff_r        <= #1  ff;

  end


  always @(posedge fpga_reset or posedge clk100) //register cypress inputs in IOFFs
  if (fpga_reset) begin
    cy_data_i   <= 16'h0;
    cy_req      <= 1'b0;
    load_state  <= 1'b0;
    state       <= 4'b0;
  end else
  begin
    cy_data_i   <= cy_data_p;
    cy_req      <= cy_req_p;
    load_state  <= load_state_p;
    state       <= state_p;
  end  

  always @(posedge fpga_reset or posedge clk100) //options register
  if (fpga_reset) begin
    op_reg   <= 16'h6054;
  end else
  begin
	  if ({load_state,state[3:0]} == {1'b1,XCOM_OPT}) begin 
	    op_reg[15:0] <= cy_data_i[15:0];
	  end
  end

  always @(posedge fpga_reset or posedge clk100) //frequency counter
  if (fpga_reset) begin
    sm_frq_cnt <= SM_FRQ_IDL;
    frq_cnt_on  <= 1'b0;
    frq_rg0     <= 16'h0000;  //TMP!!
    frq_rg1     <= 16'h00f0;  //TMP!!
    frq_cnt_dn  <= 1'b1;   //TMP!!
  end
  else begin

    case (sm_frq_cnt)

      SM_FRQ_IDL:
	    if ({load_state_p,state_p[3:0]} == {1'b1,XCOM_FRQ}) begin
	      frq_cnt_on  <= 1'b1;
	      sm_frq_cnt  <= SM_FRQ_ST1;
	    end

      SM_FRQ_ST1:
	    begin
	      frq_cnt_on <= 1'b0;
              if (frq_cnt_dn & cy_req) begin
                frq_rg0_vld <= 1'b1;
	        sm_frq_cnt  <= SM_FRQ_ST2;
	      end
	    end
	      
      SM_FRQ_ST2:
            if (~cy_req) begin
	       frq_rg0_vld <= 1'b0;
	       sm_frq_cnt  <= SM_FRQ_ST3;
	    end

      SM_FRQ_ST3:
            if (cy_req) begin
	       frq_rg1_vld <= 1'b1;
	       sm_frq_cnt  <= SM_FRQ_ST4;
	    end

      SM_FRQ_ST4:
            if (~cy_req) begin
	       frq_rg1_vld <= 1'b0;
	       sm_frq_cnt  <= SM_FRQ_IDL;
	    end

      default:
	   sm_frq_cnt  <= SM_FRQ_IDL;
    endcase
  end

  always @(posedge fpga_reset or posedge rclk) //collect
  if (fpga_reset) begin
    sm_coll  <= SM_COLL_IDL;
    wen      <= 1'b0;
    ren      <= 1'b0;
    ren1     <= 1'b0;
    ren2     <= 1'b0;
    ren3     <= 1'b0;
    ren4     <= 1'b0;
    ren5     <= 1'b0;
	ovr_ff   <= 1'b0;
	clk_led_cntr <= 25'h0;  //for convenience
  end
  else begin

    ren1    <= #1 ren;
    ren2    <= #1 ren1;
    ren3    <= #1 ren2;
    ren4    <= #1 ren3;
    ren5    <= #1 ren4;
	clk_led_cntr <= clk_led_cntr + 1'b1; //for convenience

    case (sm_coll)

      SM_COLL_IDL:
	    if ({load_state_p,state_p[3:0]} == {1'b1,XCOM_COL}) begin
          wen      <= 1'b1;
	      sm_coll  <= SM_COLL_ST1;
	    end

      SM_COLL_ST1:
	    if (ff_r) begin
          wen      <= 1'b0;
		  ovr_ff   <= ovrange;
	      sm_coll  <= SM_COLL_ST2;
	    end

      SM_COLL_ST2:
	    begin
          ren      <= #1 1'b1;
	      sm_coll  <= #1 SM_COLL_ST3;
	    end

      SM_COLL_ST3:
	    begin
          if (ef_r) begin
	        ren      <= #1 1'b0;
	        sm_coll  <= SM_COLL_ST4;
	      end
	    end

      SM_COLL_ST4:
	    begin
          if (~load_state_p) sm_coll  <= SM_COLL_IDL;
	    end

      default:
	    sm_coll  <= SM_COLL_IDL;

    endcase

  end

  always @(posedge fpga_reset or posedge clk100) //upload
  if (fpga_reset) begin
    sm_upl    <= SM_UPL_IDL;
  end
  else begin


	case (sm_upl)

      SM_UPL_IDL:
	    if ({load_state_p,state_p[3:0]} == {1'b1,XCOM_UPL}) begin
	      sm_upl  <= SM_UPL_ST1;
	    end

      SM_UPL_ST1:
	    if (cy_req) begin		//take xlf_ren active combinatorially
          fifo_q_vld  <= 1'b1;
	      sm_upl      <= SM_UPL_ST2;
	    end
	    else if ({load_state_p,state_p[3:0]} == {1'b1,XCOM_IDL}) sm_upl <= SM_UPL_IDL;

      SM_UPL_ST2:
	    if (~cy_req) begin
          fifo_q_vld  <= 1'b0;
	      sm_upl      <= SM_UPL_ST1;
	    end

      default:
	    sm_upl <= SM_UPL_IDL;

    endcase

  end

//Cypress's data bus...
  always @(frq_rg0 or frq_rg1 or fifo_q or frq_rg0_vld or frq_rg1_vld or fifo_q_vld)

    case ({frq_rg1_vld,frq_rg0_vld,fifo_q_vld})

       3'b001:    cy_data_o = fifo_q;
       3'b010:    cy_data_o = frq_rg0;
       3'b100:    cy_data_o = frq_rg1;

       default:  cy_data_o = 16'h0;
     endcase

assign cy_data_oe = cy_req_p & (frq_rg0_vld | frq_rg1_vld | fifo_q_vld);
assign cy_valid_p = cy_req_p & (frq_rg0_vld | frq_rg1_vld | fifo_q_vld);
assign cy_data_p  = cy_data_oe ? cy_data_o : 16'hZ;

//combinatorial...

assign nADC_1V9_en =  op_reg[0];
assign ecm_fsr     =  op_reg[2]  ? 1'bZ : op_reg[7];  //if spi tristate; else apply fsr;
assign sdata       =  op_reg[2]  ? mstr_sdata_p : (op_reg[5] ? 1'bZ : op_reg[4]);
                     //if spi then sdata; else if hw_ddr then tri else outedge
assign calibrate   = op_reg[8];
assign power_down  = op_reg[9];
assign dc_select   = op_reg[11];
assign ext_clk_sel = op_reg[12];
assign calDlyScs_  = op_reg[2]   ? op_reg[13] : 1'b1;  //if spi SCS_; else short cal dly
assign pll_le      = op_reg[14];

assign diff_sel    = op_reg[15];

assign sclk        = mstr_sclk_p;

assign adc_reset  =  fpga_reset;

assign dqa_p   = da_r;
assign dqa_n   = op_reg[7:0];
assign dqb_p   = db_r;
assign dqb_n   = op_reg[15:8];

/*   LEDs   */

assign trig_o      = ~trig_in;
assign upload      = ~(state_p == XCOM_UPL);
assign sample      = ~(state_p == XCOM_COL);
assign idle        = ~(state_p == XCOM_IDL);
assign clk_active  = clk_led_cntr[24];
assign ovr_led     =  ~ovr_ff;


/**** To Be Done....*/

//assign test_p     =  clk_led_cntr[3:0]; //tmp
assign test_p     =  4'hz; //tmp
//assign test_n[4:1] =  4'hZ;
assign test_n[4:1] =  {vclk,vclk,vclk,vclk};
assign strobe_q_p  = 1'b0;
assign strobe_q_n  = 1'b0;

/**** To Be Done....*/

endmodule

