module ClkBlk(fpga_reset,clk_osc,rclk_in_p,rclk_set,clk50,clk100,rclk,vclk,clk_osc_lckd);
input clk_osc,rclk_in_p,fpga_reset;
input [2:0] rclk_set;
output clk_osc_lckd,clk100,clk50,rclk,vclk;
wire dcm50_rst,clk_p100,drdy;
reg [1:0] dcm50_cntr;
reg [2:0] rclk_reg;
reg rclk_trg,dcm100_rst,dcm100_de,dcm100_we;
reg [3:0] sm_drp;
reg [15:0] dcm100_di;
reg [15:0] mul_val,div_val;
reg [6:0] dcm100_ad;

parameter [3:0]
  SM_DRP_IDL  = 4'h0,
  SM_DRP_MUL1 = 4'h1,
  SM_DRP_MUL2 = 4'h2,
  SM_DRP_MUL3 = 4'h3,
  SM_DRP_DIV1 = 4'h4,
  SM_DRP_DIV2 = 4'h5,
  SM_DRP_DIV3 = 4'h6;

dcm100 i_dcm100(.CLKIN_IN(clk_osc), 
		        .DADDR_IN(dcm100_ad), //++
				.DCLK_IN(clk100),  //++
				.DEN_IN(dcm100_de),   //++
				.DI_IN(dcm100_di),    //++
				.DWE_IN(dcm100_we),   //++
                .RST_IN(dcm100_rst), 
		        .CLKDV_OUT(clk50),
				.CLKFX_OUT(vclk), //++
                .CLKIN_IBUFG_OUT(clk_p100), //clk100 - straight from IBUFG
                .CLK0_OUT(),        //used to put out clk100
				.DO_OUT(),    //++
				.DRDY_OUT(dcm100_drdy),  //++
                .LOCKED_OUT(clk_osc_lckd)
	       );

BUFG CLKP_BUFG100(.I(clk_p100), 
                  .O(clk100));

dcm50  i_dcm50(.CLKIN_IN(rclk_in_p), 
                .RST_IN(dcm50_rst), 
                .CLKIN_IBUFG_OUT(),
                .CLK0_OUT(rclk), 
                .CLK180_OUT(), 
                .LOCKED_OUT()
	       );

always @(posedge fpga_reset or posedge clk100)  //DRP for rclk
  if (fpga_reset) begin
    rclk_reg   <= 3'h1;
	rclk_trg   <= 1'b0;
	sm_drp     <= 4'h0;
	dcm100_ad  <= 7'h0;
	dcm100_rst <= 1'b1;
	dcm100_di  <= 16'h0;
	dcm100_we  <= 1'h0;
	dcm100_de  <= 1'h0;
	sm_drp     <= SM_DRP_IDL;
  end else
  begin

    if (rclk_reg != rclk_set) begin
	  rclk_trg <= 1'b1;
	  rclk_reg <= rclk_set;
    end

    case (sm_drp)

       SM_DRP_IDL:
         begin
           dcm100_rst <= 1'b0;
		   if (rclk_trg) begin
             dcm100_rst <= 1'b1;
			 dcm100_ad  <= 7'h50;
			 sm_drp     <= SM_DRP_MUL1;
		   end
		 end

       SM_DRP_MUL1:
         begin
           rclk_trg   <= 1'b0;
           dcm100_di  <= mul_val;
           dcm100_de  <= 1'b1;
           dcm100_we  <= 1'b1;
           sm_drp     <= SM_DRP_MUL2;
		 end

       SM_DRP_MUL2:
         begin
           dcm100_de  <= 1'b0;
           dcm100_we  <= 1'b0;
           sm_drp     <= SM_DRP_MUL3;
		 end

       SM_DRP_MUL3:
        if (dcm100_drdy) sm_drp <= SM_DRP_DIV1;

       SM_DRP_DIV1:	
         begin
           dcm100_ad  <= 7'h52;
           dcm100_di  <= div_val;
           dcm100_de  <= 1'b1;
           dcm100_we  <= 1'b1;
           sm_drp     <= SM_DRP_DIV2;
		 end

       SM_DRP_DIV2:	
         begin
           dcm100_de  <= 1'b0;
           dcm100_we  <= 1'b0;
           sm_drp     <= SM_DRP_DIV3;
		 end

       SM_DRP_DIV3:
        if (dcm100_drdy) sm_drp <= SM_DRP_IDL;

       default:
         sm_drp <= SM_DRP_IDL;

    endcase
end


always @(rclk_set)

 begin
  case (rclk_set)

    3'h0:  mul_val = 16'h1;
    3'h1:  mul_val = 16'h1;
    3'h2:  mul_val = 16'h2;
    3'h3:  mul_val = 16'h1;
    3'h4:  mul_val = 16'h4;
    3'h5:  mul_val = 16'h2;
    3'h6:  mul_val = 16'h6;
    3'h7:  mul_val = 16'h1;

  endcase

  case (rclk_set)

    3'h0:  div_val = 16'h4;
    3'h1:  div_val = 16'h3;
    3'h2:  div_val = 16'h3;
    3'h3:  div_val = 16'h1;
    3'h4:  div_val = 16'h3;
    3'h5:  div_val = 16'h1;
    3'h6:  div_val = 16'h3;
    3'h7:  div_val = 16'h0;

  endcase

 end

always @(posedge fpga_reset or posedge clk100)  //reset for dcm50
  if (fpga_reset) dcm50_cntr <= 2'h0;
  else if ( ~clk_osc_lckd ) dcm50_cntr <= 2'h0;
  else if ( ~(&dcm50_cntr)) dcm50_cntr <= dcm50_cntr + 1'b1;

assign dcm50_rst = ~(&dcm50_cntr);

endmodule

